﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using EnvelopeCreatorProject.Models;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;

namespace EnvelopeCreatorProject
{
    public class DocusignTokenHandler : IDocusignTokenHandler
    {
        private string _error;
        private ILogger<DocusignTokenHandler> logger;
        private readonly string _docusignAPIVersion;
        private readonly string _docusignIntegratorKey;
        public string ErrorMessage { get; set; }

        public DocusignTokenHandler(string integratorKey, string APIVersion, ILogger<DocusignTokenHandler> logger)
        {
            _docusignAPIVersion = APIVersion;
            _docusignIntegratorKey = integratorKey;
            this.logger = logger;
        }

        //Create a Docusign token for the user name given
        public string getDocusignOauth2Token(string DocusignUsername, string DocusignPassword, string environmentURL)
        {
            string result = "";
            string url = createDocusignURL("/oauth2/token", environmentURL);
            string data = string.Format("grant_type=password&client_id={0}&username={1}&password={2}&scope=api", _docusignIntegratorKey, DocusignUsername, DocusignPassword); 
            result = makeRequest(url, "POST", "application/x-www-form-urlencoded", data);
            if (!result.Equals("ERROR"))
            {
                Dictionary<string, string> dicResult = JsonConvert.DeserializeObject<Dictionary<string, string>>(result);
                result = dicResult["access_token"];
            }
            else
            {
                result = "";
            }
            return result;
        }

        public int revokeDocusignOauth2Token(string DocusignToken, string environmentURL)
        {
            try
            {
                string url = createDocusignURL("/oauth2/revoke", environmentURL);
                string data = "token=" + DocusignToken;
                string result = makeRequest(url, "POST", "application/x-www-form-urlencoded", data);
                return result.Equals("ERROR") ? 1 : 0;
            }
            catch (Exception ex)
            {
                logger.LogError("Error while trying to revoke Docusign OAuth token.  MESSAGE: " + ex.Message + ". STACK TRACE: " + ex.StackTrace);
                return 1;
            }
        }

        private string makeRequest(string url, string type, string contentType, string data)
        {
            Byte[] byteArray = Encoding.UTF8.GetBytes(data);
            string result;
            try
            {
                var request = WebRequest.Create(url);
                request.Method = type;
                request.ContentType = contentType;
                request.ContentLength = byteArray.Length;
                request.Credentials = CredentialCache.DefaultCredentials;
                Stream requestDataStream = null;
                if (!contentType.Equals("GET"))
                {
                    using (requestDataStream)
                    {
                        requestDataStream = request.GetRequestStream();
                        requestDataStream.Write(byteArray, 0, byteArray.Length);
                    }
                }
                WebResponse response = null;
                using (response)
                {
                    request.Timeout = 60000;
                    response = request.GetResponse();
                    Stream responseDataStream = response.GetResponseStream();
                    int length = Convert.ToInt32(response.Headers["Content-Length"]);
                    BinaryReader reader = new BinaryReader(responseDataStream);
                    byteArray = reader.ReadBytes(length);
                    result = Encoding.UTF8.GetString(byteArray);
                }
            }
            catch (WebException ex)
            {
                logger.LogError("Error on WebRequest to: URL "+ url +"  MESSAGE: " + ex.Message + ". STACK TRACE: " + ex.StackTrace);
                result = "ERROR";
                if (ex.Response == null)
                {
                    ErrorMessage = ex.InnerException != null ? ex.InnerException.Message : ex.Message;
                }
                else
                {
                    string json = new StreamReader(ex.Response.GetResponseStream()).ReadToEnd();
                    Dictionary<string, string> dicResult = JsonConvert.DeserializeObject<Dictionary<string, string>>(json);
                    ErrorMessage = dicResult["error"] + "_" + dicResult["error_description"];
                }
            }

            return result;
        }

        private string createDocusignURL(string actionPath, string environmentURL)
        {
            return "https://" + environmentURL + "/restapi/" + _docusignAPIVersion + actionPath;
        }

        public DocusignAccountInformation getAccountInformation(string token, string environmentURL)
        {
            string result = "";
            string url = createDocusignURL("/login_information", environmentURL);

            try
            {
                var request = (HttpWebRequest)WebRequest.Create(url);
                request.Method = "GET";
                string authenticateStr = "bearer " + token;
                request.Headers.Add("Authorization", authenticateStr);
                request.Accept = "application/json";
                request.ContentType = "application/json";
                WebResponse response = null;
                using (response)
                {
                    request.Timeout = 60000;
                    response = request.GetResponse();
                    Stream responseDataStream = response.GetResponseStream();
                    int length = Convert.ToInt32(response.Headers["Content-Length"]);
                    BinaryReader reader = new BinaryReader(responseDataStream);
                    byte[] byteArray = reader.ReadBytes(length);
                    result = Encoding.UTF8.GetString(byteArray);
                }

            }
            catch (Exception ex)
            {
                logger.LogError("Error while trying to get Docusign Account Information.  MESSAGE: " + ex.Message + ". STACK TRACE: " + ex.StackTrace);
                _error = ex.Message;
            }

            return JsonConvert.DeserializeObject<DocusignAccountInformation>(result);
        }
    }

}
