﻿using System;
using EnvelopeCreatorProject.Interfaces;
using EnvelopeCreatorProject.Models;
using EnvelopeCreatorProject.Models.Signers;
using System.Linq;
using EnvelopeCreatorProject.Exceptions;
using Microsoft.Extensions.Logging;

namespace EnvelopeCreatorProject
{
    public class EnvelopeTransporterCreator : IEnvelopeCreator
    {
        private readonly ILogger<OAuthSigner> logger;
        private readonly ILogger<DocusignTokenHandler> dthLogger;

        private readonly string _integratorKey;
        private readonly string _APIVersion;

        public EnvelopeTransporterCreator(string integratorKey, string APIVersion, ILogger<OAuthSigner> logger, ILogger<DocusignTokenHandler> dthLogger)
        {
            _integratorKey = integratorKey;
            _APIVersion = APIVersion;
            this.logger = logger;
            this.dthLogger = dthLogger;
        }


        public EnvelopeResult CreateEnvelope(AccountAuthentication authentication, EnvelopeData EData)
        {
            try
            {
                ISigner signer = GetSigner(authentication);
                return signer.GenerateEnvelope(EData);
            }
            catch (OAuthTokenAuthorizationException ote)
            {
                return new EnvelopeResult()
                {
                    ErrorCode = ErrorCode.OAuthError,
                    Message = ote.Message
                };
            }
            catch (RecipientTypeNullException rtn)
            {
                return new EnvelopeResult()
                {
                    ErrorCode = ErrorCode.InvalidRecipientType,
                    Message = rtn.Message
                };
            }
            catch (SignerHostNullException she)
            {
                return new EnvelopeResult()
                {
                    ErrorCode = ErrorCode.InvalidSignerHost,
                    Message = she.Message
                };
            }
            catch (Exception ex)
            {
                logger.LogError(" -- CreateEnvelope error -- MESSAGE: " + ex.Message + ". STACK TRACE: " +
                             ex.StackTrace.ToString());
                return new EnvelopeResult()
                {
                    ErrorCode = ErrorCode.Unknown,
                    Message = "Unexpected error occurred. Message: " + ex.Message
                };
            }
        }

        private ISigner GetSigner(AccountAuthentication authentication)
        {
            var tokenHandler = new DocusignTokenHandler(_integratorKey, _APIVersion, dthLogger);
            var senderAccountInfo = tokenHandler.getAccountInformation(authentication.SenderToken, authentication.EnvironmentURL);
            var authAccountInfo = tokenHandler.getAccountInformation(authentication.AuthToken, authentication.EnvironmentURL);
            var authBaseURl = getAuthBaseUrl(authAccountInfo, authentication);
            var senderEmail = senderAccountInfo == null ? "" : senderAccountInfo.loginAccounts.First().email;

            return authentication.SOBOFlag ?
                (OAuthSigner)new SOBOSigner(logger) { AuthToken = authentication.AuthToken, BaseURL = authBaseURl, SenderMail = senderEmail, SenderToken = authentication.SenderToken }
                : (OAuthSigner)new StandardSigner(logger) { AuthToken = authentication.AuthToken, BaseURL = authBaseURl };
        }

        private string getAuthBaseUrl(DocusignAccountInformation authAccountInfo, AccountAuthentication authentication) {
            if(authAccountInfo == null)
                return string.Empty;
            
            var accountToUse = authAccountInfo.loginAccounts.FirstOrDefault(a => bool.TryParse(a.isDefault, out var acc) && acc);

            if (authentication.AccountID.HasValue) { 
                var account = authAccountInfo.loginAccounts.FirstOrDefault(a => a.accountId == authentication.AccountID.Value.ToString());
                if (account == null)
                    throw new OAuthTokenAuthorizationException("The DocuSign AccountID provided in this request cannot be validated for your account. Either correct it or remove it from the request so the default DocuSign account can be used.");

                return account.baseUrl;
            }
            
            return accountToUse?.baseUrl;
        }
    }
}