﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using EnvelopeCreatorProject;
using EnvelopeCreatorProject.Models;
using EnvelopeCreatorProject.Models.Signers;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using quikesigntransport.selfservice.ContractResolvers;

namespace quikesigntransport.selfservice.Services
{
    public class SignHelperService : ISignHelperService
    {
        #region Properties

        string _UserName { get; }
        string _Password { get; }
        string _QuikOAuthTokenEndpoint { get; }
        string _QuikDocusignSelfServiceEndpoint { get; }
        string _DocusignAuthtoken { get; }
        string _DocusignEnvironmentUrl { get;  }
        string _DocusignIntegratorKey { get; }
        string _DocusignAPIVersion { get; }
        ILogger<OAuthSigner> _logger { get; }
        ILogger<DocusignTokenHandler> _dthLogger { get; }

        #endregion

        #region Constructor

        public SignHelperService(string userName, string password, string quikOAuthTokenEndpoint, string quikDocusignSelfServiceEndpoint, string docusignAuthToken, string docusignEnvironmentUrl, string docusignIntegratorKey, string docusignAPIVersion, ILogger<OAuthSigner> logger, ILogger<DocusignTokenHandler> dthLogger)
        {
            this._UserName = userName;
            this._Password = password;
            this._QuikOAuthTokenEndpoint = quikOAuthTokenEndpoint;
            this._QuikDocusignSelfServiceEndpoint = quikDocusignSelfServiceEndpoint;
            this._DocusignAuthtoken = docusignAuthToken;
            this._DocusignEnvironmentUrl = docusignEnvironmentUrl;
            this._DocusignIntegratorKey = docusignIntegratorKey;
            this._DocusignAPIVersion = docusignAPIVersion;
            this._logger = logger;
            this._dthLogger = dthLogger;
        }

        #endregion

        public SelfServiceData GetEnvelopeData(string jsonString, string quikToken)
        {
            var headers = new List<KeyValuePair<string, string>>()
            {
                new KeyValuePair<string, string>("Authorization", $"BEARER { quikToken }")
            };

            var result = DoRequest(_QuikDocusignSelfServiceEndpoint, jsonString, headers);           

            var jsonResponse = JObject.Parse(result);

            var deserializerSettings = new JsonSerializerSettings()
            {
                ContractResolver = new PrivateMemberContractResolver()
            };

            if (jsonResponse["ErrorCode"].ToObject<int>() != 0)
                throw new Exception("Error getting self service data from Esignature endpoint. See the details in the response: " + result);

            var selfServiceData = JsonConvert.DeserializeObject<SelfServiceData>(jsonResponse["ResultData"].ToString(), deserializerSettings);

            return selfServiceData;
        }

        public EnvelopeResult GenerateDocusignEnvelope(SelfServiceData selfServiceData)
        {
            var envelopeCreator = new EnvelopeTransporterCreator(_DocusignIntegratorKey, _DocusignAPIVersion, _logger, _dthLogger);
            return envelopeCreator.CreateEnvelope(selfServiceData.AccountAuthentication, selfServiceData.EnvelopeData);
        }

        public string GenerateQuikToken()
        {            
            var response = DoRequest(_QuikOAuthTokenEndpoint, $"grant_type=password&username={ _UserName }&password={ _Password }", null, "application/x-www-form-urlencoded");
            return JObject.Parse(response)["access_token"].ToString();
        }

        public AccountAuthentication GetDocusignAuthentication()
        {
            return new AccountAuthentication()
            {
                AuthToken = _DocusignAuthtoken,
                SenderToken = "",
                SOBOFlag = false,
                EnvironmentURL = _DocusignEnvironmentUrl
            };
        }

        #region Private

        private string DoRequest(string url, string body, List<KeyValuePair<string, string>> headers, string contentType = "application/json")
        {
            var request = WebRequest.Create(url);
            request.ContentType = contentType;
            request.Method = "POST";

            if (headers != null)
            {
                foreach (var header in headers)
                {
                    request.Headers.Add(header.Key, header.Value);
                }
            }

            var streamBody = Encoding.UTF8.GetBytes(body);
            request.ContentLength = streamBody.Length;
            using (var requestDataStream = request.GetRequestStream())
            {
                requestDataStream.Write(streamBody, 0, streamBody.Length);
                using (var response = request.GetResponse())
                {
                    var responseDataStream = response.GetResponseStream();
                    var responseString = new StreamReader(responseDataStream).ReadToEnd();
                    return responseString;
                }
            }
        }

        #endregion
    }
}
